/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 2011 OpenFOAM Foundation
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

namespace Foam
{

// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

inline const Foam::fvMesh& Foam::solidParticleCloud::mesh() const
{
    return mesh_;
}


inline Foam::scalar Foam::solidParticleCloud::rhop() const
{
    return rhop_;
}


inline Foam::scalar Foam::solidParticleCloud::e() const
{
    return e_;
}


inline Foam::scalar Foam::solidParticleCloud::mu() const
{
    return mu_;
}


// 2Way Coupling- Momentum Source:
inline tmp<volVectorField> solidParticleCloud::momentumSource() const
{
	tmp<volVectorField> tsource
	(
	    new volVectorField
	    (
	       IOobject
	       (
	          "smom",
	          mesh_.time().timeName(),
	          mesh_,
	          IOobject::NO_READ,
	          IOobject::NO_WRITE
	       ),
	       mesh_,
	       dimensionedVector
	          (
	             "zero",
	             dimensionSet(1, -2, -2, 0, 0),
	             vector::zero
	          )
	    )
   );
   tsource().internalField() = - smom_/(mesh_.time().deltaT().value()*mesh_.V());

   return tsource;
}

inline Foam::vectorField& Foam::solidParticleCloud::smom()
{
    return smom_;
}

inline const Foam::vectorField& Foam::solidParticleCloud::smom() const
{
    return smom_;
}


inline tmp<volScalarField> solidParticleCloud::AddTOAlpha() const
{
	tmp<volScalarField> alphasource
	(
	new volScalarField
	(
	IOobject
	(
	"correctalphaW",
	mesh_.time().timeName(),
	mesh_,
	IOobject::NO_READ,
	IOobject::NO_WRITE
	),
	mesh_,
	dimensionedScalar
	(
	"zero",
	dimensionSet(0,0 , 0, 0, 0),
	0
	)
	)
	);
	alphasource().internalField() = correctalphaW_;
	return alphasource;
	}
	
	
	inline tmp<volVectorField> solidParticleCloud::AddToU() const
	{
		tmp<volVectorField> Usource
		(
		new volVectorField
		(
		IOobject
		(
		"correctU",
		mesh_.time().timeName(),
		mesh_,
		IOobject::NO_READ,
		IOobject::NO_WRITE
		),
		mesh_,
		dimensionedVector
		(
		"correctU",
		dimensionSet(0, 1, -1, 0, 0),
		vector(0,0,0)
		)
		)
		);
		Usource().internalField() = correctU_;
		return Usource;
		}
		
		
inline Foam::scalarField& Foam::solidParticleCloud::correctalphaW()
{
return correctalphaW_;
}

inline const Foam::scalarField& Foam::solidParticleCloud::correctalphaW() const
{
return correctalphaW_;
}

inline Foam::vectorField& Foam::solidParticleCloud::correctU()
{
return correctU_;
}

inline const Foam::vectorField& Foam::solidParticleCloud::correctU() const
{
return correctU_;
}

inline Foam::scalar Foam::solidParticleCloud::p0() const
{
return p0_;
}
inline Foam::scalar Foam::solidParticleCloud::pSat() const
{
return pSat_;
}
inline Foam::scalar Foam::solidParticleCloud::gamma() const
{
return gamma_;
}
inline Foam::scalar Foam::solidParticleCloud::sigma() const
{
return sigma_;
}

}   // End namespace Foam
// ************************************************************************* //
