/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 2011-2015 OpenFOAM Foundation
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "solidParticle.H"
#include "IOstreams.H"

// * * * * * * * * * * * * * * Static Data Members * * * * * * * * * * * * * //

const std::size_t Foam::solidParticle::sizeofFields_
(
    sizeof(solidParticle) - sizeof(particle)
);


// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

Foam::solidParticle::solidParticle
(
    const polyMesh& mesh,
    Istream& is,
    bool readFields
)
:
    particle(mesh, is, readFields)
{
    if (readFields)
    {
        if (is.format() == IOstream::ASCII)
        {
            d_ = readScalar(is);
            is >> U_ >> Rdt_ >> R_ >> R0_ >> pB_ >> pG_ >> pG0_ >> F0Old_ >> F1Old_;
        }
        else
        {
            is.read(reinterpret_cast<char*>(&d_),
sizeof(d_)
+ sizeof(U_)
+ sizeof(Rdt_)
+ sizeof(R_)
+ sizeof(R0_)
+ sizeof(pB_)
+ sizeof(pG_)
+ sizeof(pG0_)
+ sizeof(F0Old_)
+ sizeof(F1Old_));
        }
    }

    // Check state of Istream
    is.check("solidParticle::solidParticle(Istream&)");
}


void Foam::solidParticle::readFields(Cloud<solidParticle>& c)
{
    if (!c.size())
    {
        return;
    }

    particle::readFields(c);

    IOField<scalar> d(c.fieldIOobject("d", IOobject::MUST_READ));
    c.checkFieldIOobject(c, d);

    IOField<vector> U(c.fieldIOobject("U", IOobject::MUST_READ));
    c.checkFieldIOobject(c, U);
    
    IOField<scalar> Rdt(c.fieldIOobject("Rdt", IOobject::MUST_READ));
c.checkFieldIOobject(c, Rdt);
IOField<scalar> R(c.fieldIOobject("R", IOobject::MUST_READ));
c.checkFieldIOobject(c, R);
IOField<scalar> R0(c.fieldIOobject("R0", IOobject::MUST_READ));
c.checkFieldIOobject(c, R0);
IOField<scalar> pB(c.fieldIOobject("pB", IOobject::MUST_READ));
c.checkFieldIOobject(c, pB);
IOField<scalar> pG(c.fieldIOobject("pG", IOobject::MUST_READ));
c.checkFieldIOobject(c, pG);
IOField<scalar> pG0(c.fieldIOobject("pG0", IOobject::MUST_READ));
c.checkFieldIOobject(c, pG0);
IOField<scalar> F0Old(c.fieldIOobject("F0Old", IOobject::MUST_READ));
c.checkFieldIOobject(c, F0Old);
IOField<scalar> F1Old(c.fieldIOobject("F1Old", IOobject::MUST_READ));
c.checkFieldIOobject(c, F1Old);

    label i = 0;
    forAllIter(Cloud<solidParticle>, c, iter)
    {
        solidParticle& p = iter();

        p.d_ = d[i];
        p.U_ = U[i];
        p.Rdt_ = Rdt[i];
p.R_ = R[i];
p.R0_ = R0[i];
p.pB_ = pB[i];
p.pG_ = pG[i];
p.pG0_ = pG0[i];
p.F0Old_ = F0Old[i];
p.F1Old_ = F1Old[i];
        i++;
    }
}


void Foam::solidParticle::writeFields(const Cloud<solidParticle>& c)
{
    particle::writeFields(c);

    label np = c.size();

    IOField<scalar> d(c.fieldIOobject("d", IOobject::NO_READ), np);
    IOField<vector> U(c.fieldIOobject("U", IOobject::NO_READ), np);
     IOField<scalar> Rdt(c.fieldIOobject("Rdt", IOobject::NO_READ), np);
       IOField<scalar> R(c.fieldIOobject("R", IOobject::NO_READ), np);
       IOField<scalar> R0(c.fieldIOobject("R0", IOobject::NO_READ), np);
       IOField<scalar> pB(c.fieldIOobject("pB", IOobject::NO_READ), np);
       IOField<scalar> pG(c.fieldIOobject("pG", IOobject::NO_READ), np);
       IOField<scalar> pG0(c.fieldIOobject("pG0", IOobject::NO_READ), np);
       IOField<scalar> F0Old(c.fieldIOobject("F0Old", IOobject::NO_READ), np);
       IOField<scalar> F1Old(c.fieldIOobject("F1Old", IOobject::NO_READ), np);

    label i = 0;
    forAllConstIter(Cloud<solidParticle>, c, iter)
    {
        const solidParticle& p = iter();

        d[i] = p.d_;
        U[i] = p.U_;
        Rdt[i] = p.Rdt_;
R[i] = p.R_;
R0[i] = p.R0_;
pB[i] = p.pB_;
pG[i] = p.pG_;
pG0[i] = p.pG0_;
F0Old[i] = p.F0Old_;
F1Old[i] =p.F1Old_;
        i++;
    }

    d.write();
    U.write();
    Rdt.write();
R.write();
R0.write();
pB.write();
pG.write();
pG0.write();
F0Old.write();
F1Old.write();
}


// * * * * * * * * * * * * * * * IOstream Operators  * * * * * * * * * * * * //

Foam::Ostream& Foam::operator<<(Ostream& os, const solidParticle& p)
{
    if (os.format() == IOstream::ASCII)
    {
        os  << static_cast<const particle&>(p)
            << token::SPACE << p.d_
            << token::SPACE << p.U_
            << token::SPACE << p.Rdt_
         << token::SPACE << p.R_
         << token::SPACE << p.R0_
         << token::SPACE << p.pB_
         << token::SPACE << p.pG_
         << token::SPACE << p.pG0_
         << token::SPACE << p.F0Old_
         << token::SPACE << p.F1Old_;
    }
    else
    {
        os  << static_cast<const particle&>(p);
        os.write
        (
            reinterpret_cast<const char*>(&p.d_),
             sizeof(p.d_) 
        + sizeof(p.U_)
        + sizeof(p.Rdt_)
        + sizeof(p.R_)
        + sizeof(p.R0_)
        + sizeof(p.pB_)
        + sizeof(p.pG_)
        + sizeof(p.pG0_)
        + sizeof(p.F0Old_)
        + sizeof(p.F1Old_)
        );
    }

    // Check state of Ostream
    os.check("Ostream& operator<<(Ostream&, const solidParticle&)");

    return os;
}


// ************************************************************************* //
